/* 
 * ==========================================================
 * Artekit demo sketch for the AK-MPL3115A2 Pressure/Temperature
 * sensor breakout board.
 *
 * Copyright (c) 2013 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-MPL3115A2
 * ==================================
 *   AN5                   SCL
 *   AN4                   SDA
 *   3.3V                  VCC
 *   GND                   GND
 *
 * You may need a level converter for the I2C lines, since
 * the Arduino moves these lines from 0V to 5V, and the
 * AK-MPL3115A2 works only with 3.3V
 *
 * ==========================================================
 */
 
#include <Wire.h>

#define MPL3115A2_ADDR  0x60
 
/* This function reads a byte for the given register */
static byte i2cRead(byte reg)
{
  Wire.beginTransmission(MPL3115A2_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(MPL3115A2_ADDR, 1);
  return Wire.read();
}
 
/* This function writes a byte to the given register */
static void i2cWrite(byte reg, byte value)
{
  Wire.beginTransmission(MPL3115A2_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

static void sensorConfig()
{
  /* Altimeter mode, oversampling 512 ms */
  i2cWrite(0x26, 0xB9);  /* CTRL_REG1 */
  
  /* Data ready event and temperature change interrupts enabled */
  i2cWrite(0x29, 0x81);  /* CTRL_REG4 */
  
  /* Data ready event interrupt on pin INT1.
   * Temperature change interrupt on pin INT2.
   * This sketch doesn't use these interrupts. I'm enabling
   * them as an example and just to see them with the scope.
   */
  i2cWrite(0x2A, 0x80);  /* CTRL_REG5 */
  
  /* Enable data ready event for both altitude and temperature */
  i2cWrite(0x13, 0x07);  /* PT_DATA_CFG */

  /* 
   * Change these values for your location hPa.
   * Porto Recanati weather station barometer: 1012.7 hPa
   * so 101270 Pa / 2 = 50635 = 0xC6CB
   */
  i2cWrite(0x14, 0xC6);
  i2cWrite(0x15, 0xCB);
}

static bool sensorHasNewData()
{
  /* Return true whenever there is a new altitude or temperature change */
  if(i2cRead(0x12) & 0x81)
  {
      return true;
  }
  
  return false;
}

static float sensorReadTemperature()
{
  int temp_msb;
  float temperature, temp_lsb;
  
  temp_msb = i2cRead(0x04);
  temp_lsb = (float) (i2cRead(0x05) >> 4);
  
  temperature = temp_msb;
  temperature += temp_lsb / 16;
  
  return temperature;
}

static float sensorReadAltitude()
{
  int alt_msb, alt_csb;  
  float altitude, alt_lsb, temp_lsb;
  
  /* Altitude is split over 3 registers (0x01, 0x02, and 0x03).
   * The first two contain the integer part. The last one, the decimal part.
   */ 
  alt_msb = i2cRead(0x01);
  alt_csb = i2cRead(0x02);
  alt_lsb = (float) (i2cRead(0x03) >> 4);
  
  altitude = alt_msb << 8;
  altitude += alt_csb;
  altitude += alt_lsb / 16;
  return altitude;
}

static void sensorRead()
{
  /* The sensor is configured to flag events on a new altitude
   * or temperature value.
   */
  if (sensorHasNewData() == false)
  {
    /* No new data. Do nothing */
    return;
  }

  /* Print the received values */
  Serial.print("Alt=");
  Serial.print(sensorReadAltitude());
  Serial.print(" Temp=");
  Serial.println(sensorReadTemperature());
}

void setup()
{
  Wire.begin();
  Serial.begin(9600);
  sensorConfig();  
}

void loop()
{
  sensorRead();
}
